@@ -14,13 +14,15 @@
  * SOFTWARE.
  */
 
-//! Utilities for operating on context.
+//! `FEEL` context.
 
 use self::errors::*;
 use crate::names::{Name, QualifiedName};
+use crate::strings::ToFeelString;
 use crate::types::FeelType;
+use crate::value_null;
 use crate::values::Value;
-use dmntk_common::{DmntkError, Feelify, Jsonify, Stringify};
+use dmntk_common::{DmntkError, Jsonify};
 use std::collections::{BTreeMap, HashSet};
 use std::convert::TryFrom;
 use std::ops::Deref;
@@ -29,16 +31,9 @@ use std::ops::Deref;
 type FeelContextEntries = BTreeMap<Name, Value>;
 
 /// The `FEEL` context.
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq, Default)]
 pub struct FeelContext(FeelContextEntries);
 
-impl Default for FeelContext {
-  /// Creates a default empty [FeelContext].
-  fn default() -> Self {
-    Self(FeelContextEntries::new())
-  }
-}
-
 impl Deref for FeelContext {
   type Target = FeelContextEntries;
   fn deref(&self) -> &Self::Target {
@@ -65,24 +60,41 @@ impl From<FeelContext> for Value {
   }
 }
 
-impl Stringify for FeelContext {
-  /// Converts this [FeelContext] into its `TEXT` representation.
-  fn stringify(&self) -> String {
-    self.feelify()
+impl std::fmt::Display for FeelContext {
+  ///
+  fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    write!(
+      f,
+      "{{{}}}",
+      self
+        .0
+        .iter()
+        .map(|(name, value)| { format!(r#"{}: {}"#, name, value) })
+        .collect::<Vec<String>>()
+        .join(", ")
+    )
   }
 }
 
-impl Feelify for FeelContext {
-  /// Converts this [FeelContext] into its `FEEL` representation.
-  fn feelify(&self) -> String {
+impl ToFeelString for FeelContext {
+  /// Converts [FeelContext] into `FEEL` string.
+  fn to_feel_string(&self) -> String {
     format!(
       "{{{}}}",
       self
         .0
         .iter()
-        .map(|(name, value)| format!(r#"{}:{}"#, name.to_string(), value.feelify()))
+        .map(|(name, value)| {
+          let name_str = format!("{}", name);
+          let padded_name_str = match name_str.as_str() {
+            "{" | "}" | ":" | "," => format!("\"{}\"", name_str),
+            "\"" => "\"\\\"\"".to_string(),
+            _ => name_str,
+          };
+          format!(r#"{}: {}"#, padded_name_str, value)
+        })
         .collect::<Vec<String>>()
-        .join(",")
+        .join(", ")
     )
   }
 }
@@ -95,9 +107,9 @@ impl Jsonify for FeelContext {
       self
         .0
         .iter()
-        .map(|(name, value)| format!(r#""{}":{}"#, name.to_string(), value.jsonify()))
+        .map(|(name, value)| format!(r#""{}": {}"#, name, value.jsonify()))
         .collect::<Vec<String>>()
-        .join(",")
+        .join(", ")
     )
   }
 }
@@ -105,7 +117,7 @@ impl Jsonify for FeelContext {
 impl FeelContext {
   /// Checks if this [FeelContext] contains an entry pointed by [Name].
   pub fn contains_entry(&self, name: &Name) -> bool {
-    self.0.contains_key(&name)
+    self.0.contains_key(name)
   }
   /// Checks if this [FeelContext] contains an entry pointed by [QualifiedName](crate::names::QualifiedName).
   pub fn contains_entries(&self, qname: &QualifiedName) -> bool {
@@ -115,10 +127,18 @@ impl FeelContext {
   pub fn set_entry(&mut self, name: &Name, value: Value) {
     self.0.insert(name.clone(), value);
   }
+  /// Sets a null value for specified entry name in this [FeelContext].
+  pub fn set_null(&mut self, name: Name) {
+    self.0.insert(name, value_null!());
+  }
   /// Returns a value of an entry specified by name.
   pub fn get_entry(&self, name: &Name) -> Option<&Value> {
     self.0.get(name)
   }
+  /// Returns a list of key-value pairs.
+  pub fn get_entries(&self) -> Vec<(&Name, &Value)> {
+    self.0.iter().collect::<Vec<(&Name, &Value)>>()
+  }
   /// Returns a first value contained by context.
   pub fn get_first(&self) -> Option<&Value> {
     self.0.values().take(1).next()
@@ -137,6 +157,14 @@ impl FeelContext {
       self.0.insert(name.clone(), value.clone());
     }
   }
+  ///
+  pub fn overrwrite(&mut self, other: &FeelContext) {
+    for (name, value) in &other.0 {
+      if self.0.contains_key(name) {
+        self.0.insert(name.clone(), value.clone());
+      }
+    }
+  }
   /// Creates an entry with a value for specified [QualifiedName](crate::names::QualifiedName).
   /// All non existing intermediary contexts will be created.
   pub fn create_entry(&mut self, qname: &QualifiedName, value: Value) {
@@ -152,28 +180,28 @@ impl FeelContext {
         if !sub_keys.is_empty() {
           for sub_key in sub_keys {
             keys.insert(sub_key.clone());
-            keys.insert(format!("{} . {}", key.to_string(), sub_key));
+            keys.insert(format!("{} . {}", key, sub_key));
           }
         }
       }
       if let Value::List(items) = value {
-        for item in items.to_vec() {
+        for item in items.as_vec() {
           if let Value::Context(item_ctx) = item {
             let sub_keys = item_ctx.flatten_keys();
             if !sub_keys.is_empty() {
               for sub_key in sub_keys {
                 keys.insert(sub_key.clone());
-                keys.insert(format!("{} . {}", key.to_string(), sub_key)); //TODO add test for this case, nothing happened after adding spaces
+                keys.insert(format!("{} . {}", key, sub_key)); //TODO add test for this case, nothing happened after adding spaces
               }
             }
           }
         }
       }
-      if let Value::Type(FeelType::Context(a)) = value {
+      if let Value::FeelType(FeelType::Context(a)) = value {
         for name in a.keys() {
           let sub_key = name.to_string();
           keys.insert(sub_key.clone());
-          keys.insert(format!("{} . {}", key.to_string(), sub_key)); //TODO add test for this case, nothing happened after adding spaces
+          keys.insert(format!("{} . {}", key, sub_key)); //TODO add test for this case, nothing happened after adding spaces
         }
       }
     }
@@ -244,7 +272,7 @@ impl FeelContext {
 /// Definitions of context errors.
 pub mod errors {
   use crate::values::Value;
-  use dmntk_common::{DmntkError, Feelify};
+  use dmntk_common::DmntkError;
 
   /// Context errors.
   #[derive(Debug, PartialEq)]
@@ -270,7 +298,7 @@ pub mod errors {
   }
 
   pub fn value_is_not_a_context(value: &Value) -> DmntkError {
-    ContextError::ValueIsNotAContext(value.feelify()).into()
+    ContextError::ValueIsNotAContext(value.to_string()).into()
   }
 }
 
@@ -279,41 +307,72 @@ mod tests {
   use crate::context::FeelContext;
   use crate::names::{Name, QualifiedName};
   use crate::values::Value;
-  use dmntk_common::{Feelify, Jsonify};
+  use crate::{value_number, FeelNumber, ToFeelString};
+  use dmntk_common::Jsonify;
 
   #[test]
   fn test_context_default() {
     let ctx: FeelContext = Default::default();
-    assert_eq!("{}", ctx.feelify());
+    assert_eq!("{}", ctx.to_string());
     assert!(ctx.flatten_keys().is_empty());
   }
 
   #[test]
-  fn test_context_stringify() {
+  fn test_context_to_string() {
     let name_a = Name::from("a");
     let name_x_y = Name::new(&["x", "y"]);
     let name_k_plus_l_minus_m = Name::new(&["k", "+", "l", "-", "m"]);
     let mut ctx: FeelContext = Default::default();
-    ctx.set_entry(&name_a, Value::Number(10.0));
-    assert_eq!(r#"{a:10}"#, ctx.feelify());
+    ctx.set_entry(&name_a, value_number!(10));
+    assert_eq!(r#"{a: 10}"#, ctx.to_string());
     ctx.set_entry(&name_x_y, Value::Boolean(true));
-    assert_eq!(r#"{a:10,x y:true}"#, ctx.feelify());
+    assert_eq!(r#"{a: 10, x y: true}"#, ctx.to_string());
     ctx.set_entry(&name_k_plus_l_minus_m, Value::String("KLM".to_string()));
-    assert_eq!(r#"{a:10,k + l - m:"KLM",x y:true}"#, ctx.feelify());
+    assert_eq!(r#"{a: 10, k+l-m: "KLM", x y: true}"#, ctx.to_string());
   }
 
   #[test]
-  fn test_context_jsonify() {
+  fn test_context_to_feel_string() {
     let name_a = Name::from("a");
     let name_x_y = Name::new(&["x", "y"]);
     let name_k_plus_l_minus_m = Name::new(&["k", "+", "l", "-", "m"]);
     let mut ctx: FeelContext = Default::default();
-    ctx.set_entry(&name_a, Value::Number(10.0));
-    assert_eq!(r#"{"a":10}"#, ctx.jsonify());
+    ctx.set_entry(&name_a, value_number!(10));
+    assert_eq!(r#"{a: 10}"#, ctx.to_feel_string());
     ctx.set_entry(&name_x_y, Value::Boolean(true));
-    assert_eq!(r#"{"a":10,"x y":true}"#, ctx.jsonify());
+    assert_eq!(r#"{a: 10, x y: true}"#, ctx.to_feel_string());
     ctx.set_entry(&name_k_plus_l_minus_m, Value::String("KLM".to_string()));
-    assert_eq!(r#"{"a":10,"k + l - m":"KLM","x y":true}"#, ctx.jsonify());
+    assert_eq!(r#"{a: 10, k+l-m: "KLM", x y: true}"#, ctx.to_feel_string());
+    let mut ctx: FeelContext = Default::default();
+    let name_left_bracket = Name::from("{");
+    ctx.set_entry(&name_left_bracket, value_number!(1));
+    assert_eq!(r#"{"{": 1}"#, ctx.to_feel_string());
+    let name_right_bracket = Name::from("}");
+    ctx.set_entry(&name_right_bracket, value_number!(2));
+    assert_eq!(r#"{"{": 1, "}": 2}"#, ctx.to_feel_string());
+    let name_colon = Name::from(":");
+    ctx.set_entry(&name_colon, value_number!(3));
+    assert_eq!(r#"{":": 3, "{": 1, "}": 2}"#, ctx.to_feel_string());
+    let name_comma = Name::from(",");
+    ctx.set_entry(&name_comma, value_number!(4));
+    assert_eq!(r#"{",": 4, ":": 3, "{": 1, "}": 2}"#, ctx.to_feel_string());
+    let name_double_quote = Name::from("\"");
+    ctx.set_entry(&name_double_quote, value_number!(5));
+    assert_eq!(r#"{"\"": 5, ",": 4, ":": 3, "{": 1, "}": 2}"#, ctx.to_feel_string());
+  }
+
+  #[test]
+  fn test_context_to_json() {
+    let name_a = Name::from("a");
+    let name_x_y = Name::new(&["x", "y"]);
+    let name_k_plus_l_minus_m = Name::new(&["k", "+", "l", "-", "m"]);
+    let mut ctx: FeelContext = Default::default();
+    ctx.set_entry(&name_a, value_number!(10));
+    assert_eq!(r#"{"a": 10}"#, ctx.jsonify());
+    ctx.set_entry(&name_x_y, Value::Boolean(true));
+    assert_eq!(r#"{"a": 10, "x y": true}"#, ctx.jsonify());
+    ctx.set_entry(&name_k_plus_l_minus_m, Value::String("KLM".to_string()));
+    assert_eq!(r#"{"a": 10, "k+l-m": "KLM", "x y": true}"#, ctx.jsonify());
   }
 
   #[test]
@@ -325,8 +384,8 @@ mod tests {
     let qname_a_b = QualifiedName::new(&[&name_a_b]);
     let qname_a_b_c = QualifiedName::new(&[&name_a_b_c]);
     let mut ctx_a: FeelContext = Default::default();
-    ctx_a.set_entry(&name_a, Value::Number(10.0));
-    assert_eq!("{a:10}", ctx_a.feelify());
+    ctx_a.set_entry(&name_a, value_number!(10));
+    assert_eq!("{a: 10}", ctx_a.to_string());
     assert!(ctx_a.contains_entry(&name_a));
     assert!(ctx_a.contains_entries(&qname_a));
     assert!(!ctx_a.contains_entry(&name_a_b));
@@ -334,7 +393,7 @@ mod tests {
     assert!(!ctx_a.contains_entry(&name_a_b_c));
     assert!(!ctx_a.contains_entries(&qname_a_b_c));
     assert!(ctx_a.flatten_keys().contains("a"));
-    assert_eq!("10", ctx_a.get_entry(&name_a).unwrap().feelify().as_str());
+    assert_eq!("10", ctx_a.get_entry(&name_a).unwrap().to_string().as_str());
   }
 
   #[test]
@@ -350,14 +409,14 @@ mod tests {
     let qname_b_married_age = QualifiedName::new(&[&name_b, &name_married, &name_age]);
     let mut ctx_b: FeelContext = Default::default();
     ctx_b.set_entry(&name_married, Value::Boolean(true));
-    assert_eq!("{married:true}", ctx_b.feelify());
+    assert_eq!("{married: true}", ctx_b.to_string());
     assert!(ctx_b.contains_entry(&name_married));
     assert!(ctx_b.contains_entries(&qname_married));
     let mut ctx_a: FeelContext = Default::default();
-    ctx_a.set_entry(&name_age, Value::Number(49.0));
+    ctx_a.set_entry(&name_age, value_number!(49));
     ctx_a.set_entry(&name_x_y, Value::Boolean(true));
     ctx_a.set_entry(&name_b, ctx_b.into());
-    assert_eq!("{age:49,b:{married:true},x y:true}", ctx_a.feelify());
+    assert_eq!("{age: 49, b: {married: true}, x y: true}", ctx_a.to_string());
     assert!(ctx_a.contains_entry(&name_age));
     assert!(ctx_a.contains_entry(&name_b));
     assert!(ctx_a.contains_entry(&name_x_y));
@@ -368,8 +427,8 @@ mod tests {
     assert!(ctx_a.flatten_keys().contains("age"));
     assert!(ctx_a.flatten_keys().contains("b"));
     assert!(ctx_a.flatten_keys().contains("b . married"));
-    assert_eq!("49", ctx_a.get_entry(&name_age).unwrap().feelify().as_str());
-    assert_eq!("{married:true}", ctx_a.get_entry(&name_b).unwrap().feelify().as_str());
+    assert_eq!("49", ctx_a.get_entry(&name_age).unwrap().to_string().as_str());
+    assert_eq!("{married: true}", ctx_a.get_entry(&name_b).unwrap().to_string().as_str());
   }
 
   #[test]
@@ -381,18 +440,18 @@ mod tests {
     let name_c = Name::from("c");
     let mut ctx_c: FeelContext = Default::default();
     ctx_c.set_entry(&name_car, Value::String("opel".to_string()));
-    assert_eq!(r#"{car:"opel"}"#, ctx_c.feelify());
+    assert_eq!(r#"{car: "opel"}"#, ctx_c.to_string());
     assert!(ctx_c.contains_entry(&name_car));
     let mut ctx_b: FeelContext = Default::default();
     ctx_b.set_entry(&name_married, Value::Boolean(true));
     ctx_b.set_entry(&name_c, ctx_c.into());
-    assert_eq!(r#"{c:{car:"opel"},married:true}"#, ctx_b.feelify());
+    assert_eq!(r#"{c: {car: "opel"}, married: true}"#, ctx_b.to_string());
     assert!(ctx_b.contains_entry(&name_married));
     assert!(ctx_b.contains_entry(&name_c));
     let mut ctx_a: FeelContext = Default::default();
-    ctx_a.set_entry(&name_age, Value::Number(49.0));
+    ctx_a.set_entry(&name_age, value_number!(49));
     ctx_a.set_entry(&name_b, ctx_b.into());
-    assert_eq!(r#"{age:49,b:{c:{car:"opel"},married:true}}"#, ctx_a.feelify());
+    assert_eq!(r#"{age: 49, b: {c: {car: "opel"}, married: true}}"#, ctx_a.to_string());
     assert!(ctx_a.contains_entry(&name_age));
     assert!(ctx_a.contains_entry(&name_b));
     assert!(ctx_a.flatten_keys().contains("age"));
@@ -410,14 +469,14 @@ mod tests {
     let mut ctx_b: FeelContext = Default::default();
     ctx_b.set_entry(&name_married, Value::Boolean(true));
     let mut ctx_a: FeelContext = Default::default();
-    ctx_a.set_entry(&name_age, Value::Number(49.0));
+    ctx_a.set_entry(&name_age, value_number!(49));
     ctx_a.set_entry(&name_b, ctx_b.into());
     let qn_empty = QualifiedName::new(&[]);
-    assert_eq!(None, ctx_a.search_entry(&qn_empty));
+    assert!(ctx_a.search_entry(&qn_empty).is_none());
     let qn_b = QualifiedName::new(&[&name_b]);
-    assert_eq!("{married:true}", ctx_a.search_entry(&qn_b).unwrap().feelify().as_str());
+    assert_eq!("{married: true}", ctx_a.search_entry(&qn_b).unwrap().to_string().as_str());
     let qn_b_married = QualifiedName::new(&[&name_b, &name_married]);
-    assert_eq!("true", ctx_a.search_entry(&qn_b_married).unwrap().feelify().as_str());
+    assert_eq!("true", ctx_a.search_entry(&qn_b_married).unwrap().to_string().as_str());
   }
 
   #[test]
@@ -442,15 +501,15 @@ mod tests {
     // prepare contexts
     let mut ctx_c: FeelContext = Default::default();
     ctx_c.set_entry(&name_e, Value::String("e".to_string()));
-    assert_eq!(r#"{e:"e"}"#, ctx_c.feelify());
+    assert_eq!(r#"{e: "e"}"#, ctx_c.to_string());
     let mut ctx_b: FeelContext = Default::default();
     ctx_b.set_entry(&name_c, Value::String("c".to_string()));
     ctx_b.set_entry(&name_d, ctx_c.into());
-    assert_eq!(r#"{c:"c",d:{e:"e"}}"#, ctx_b.feelify());
+    assert_eq!(r#"{c: "c", d: {e: "e"}}"#, ctx_b.to_string());
     let mut ctx_a: FeelContext = Default::default();
     ctx_a.set_entry(&name_a, Value::String("a".to_string()));
     ctx_a.set_entry(&name_b, ctx_b.into());
-    assert_eq!(r#"{a:"a",b:{c:"c",d:{e:"e"}}}"#, ctx_a.feelify());
+    assert_eq!(r#"{a: "a", b: {c: "c", d: {e: "e"}}}"#, ctx_a.to_string());
     // test searching entries
     assert!(ctx_a.contains_entries(&qn_a));
     assert!(ctx_a.contains_entries(&qn_b));
@@ -480,21 +539,21 @@ mod tests {
     let qn_a_b_c_d = QualifiedName::new(&[&name_a, &name_b, &name_c, &name_d]);
     let mut ctx: FeelContext = Default::default();
     ctx.create_entry(&qn_a_b_c_d, Value::Boolean(true));
-    assert_eq!("{a:{b:{c:{d:true}}}}", ctx.feelify().as_str());
-    assert_eq!("{b:{c:{d:true}}}", ctx.search_entry(&qn_a).unwrap().feelify().as_str());
-    assert_eq!("{c:{d:true}}", ctx.search_entry(&qn_a_b).unwrap().feelify().as_str());
-    assert_eq!("{d:true}", ctx.search_entry(&qn_a_b_c).unwrap().feelify().as_str());
-    assert_eq!("true", ctx.search_entry(&qn_a_b_c_d).unwrap().feelify().as_str());
+    assert_eq!("{a: {b: {c: {d: true}}}}", ctx.to_string().as_str());
+    assert_eq!("{b: {c: {d: true}}}", ctx.search_entry(&qn_a).unwrap().to_string().as_str());
+    assert_eq!("{c: {d: true}}", ctx.search_entry(&qn_a_b).unwrap().to_string().as_str());
+    assert_eq!("{d: true}", ctx.search_entry(&qn_a_b_c).unwrap().to_string().as_str());
+    assert_eq!("true", ctx.search_entry(&qn_a_b_c_d).unwrap().to_string().as_str());
     let mut ctx: FeelContext = Default::default();
     ctx.create_entry(&qn_a, Value::Boolean(true));
     ctx.create_entry(&qn_b, Value::Boolean(false));
     ctx.create_entry(&qn_c_d, Value::String("deep".to_string()));
-    assert_eq!(r#"{a:true,b:false,c:{d:"deep"}}"#, ctx.feelify().as_str());
+    assert_eq!(r#"{a: true, b: false, c: {d: "deep"}}"#, ctx.to_string().as_str());
     let mut ctx: FeelContext = Default::default();
     ctx.create_entry(&qn_a_b, Value::String("b".to_string()));
     ctx.create_entry(&qn_a_c, Value::String("c".to_string()));
     ctx.create_entry(&qn_a_d, Value::String("d".to_string()));
-    assert_eq!(r#"{a:{b:"b",c:"c",d:"d"}}"#, ctx.feelify().as_str());
+    assert_eq!(r#"{a: {b: "b", c: "c", d: "d"}}"#, ctx.to_string().as_str());
   }
 
   #[test]
@@ -502,9 +561,9 @@ mod tests {
     let name_a = Name::from("a");
     let name_b = Name::from("b");
     let mut ctx: FeelContext = Default::default();
-    ctx.set_entry(&name_a, Value::Number(1.0));
-    ctx.set_entry(&name_b, Value::Number(2.0));
-    assert_eq!(r#"{a:1,b:2}"#, ctx.feelify());
+    ctx.set_entry(&name_a, value_number!(1));
+    ctx.set_entry(&name_b, value_number!(2));
+    assert_eq!(r#"{a: 1, b: 2}"#, ctx.to_string());
     let keys = ctx.flatten_keys();
     assert_eq!(2, keys.len());
     assert!(keys.contains("a"));
@@ -517,12 +576,12 @@ mod tests {
     let name_b = Name::from("b");
     let name_c = Name::from("c");
     let mut ctx_b: FeelContext = Default::default();
-    ctx_b.set_entry(&name_a, Value::Number(10.0));
-    ctx_b.set_entry(&name_b, Value::Number(20.0));
-    ctx_b.set_entry(&name_c, Value::Number(30.0));
+    ctx_b.set_entry(&name_a, value_number!(10));
+    ctx_b.set_entry(&name_b, value_number!(20));
+    ctx_b.set_entry(&name_c, value_number!(30));
     let mut ctx_a: FeelContext = Default::default();
-    ctx_a.set_entry(&name_a, Value::Number(1.0));
-    ctx_a.set_entry(&name_b, Value::Number(2.0));
+    ctx_a.set_entry(&name_a, value_number!(1));
+    ctx_a.set_entry(&name_b, value_number!(2));
     ctx_a.set_entry(&name_c, ctx_b.into());
     let keys = ctx_a.flatten_keys();
     assert_eq!(6, keys.len());
@@ -541,18 +600,18 @@ mod tests {
     let name_c = Name::from("c");
     let name_d = Name::from("d");
     let mut ctx_c: FeelContext = Default::default();
-    ctx_c.set_entry(&name_a, Value::Number(100.0));
-    ctx_c.set_entry(&name_b, Value::Number(200.0));
-    ctx_c.set_entry(&name_c, Value::Number(300.0));
-    ctx_c.set_entry(&name_d, Value::Number(400.0));
+    ctx_c.set_entry(&name_a, value_number!(100));
+    ctx_c.set_entry(&name_b, value_number!(200));
+    ctx_c.set_entry(&name_c, value_number!(300));
+    ctx_c.set_entry(&name_d, value_number!(400));
     let mut ctx_b: FeelContext = Default::default();
-    ctx_b.set_entry(&name_a, Value::Number(10.0));
-    ctx_b.set_entry(&name_b, Value::Number(20.0));
-    ctx_b.set_entry(&name_c, Value::Number(30.0));
+    ctx_b.set_entry(&name_a, value_number!(10));
+    ctx_b.set_entry(&name_b, value_number!(20));
+    ctx_b.set_entry(&name_c, value_number!(30));
     ctx_b.set_entry(&name_d, ctx_c.into());
     let mut ctx_a: FeelContext = Default::default();
-    ctx_a.set_entry(&name_a, Value::Number(1.0));
-    ctx_a.set_entry(&name_b, Value::Number(2.0));
+    ctx_a.set_entry(&name_a, value_number!(1));
+    ctx_a.set_entry(&name_b, value_number!(2));
     ctx_a.set_entry(&name_c, ctx_b.into());
     let keys = ctx_a.flatten_keys();
     assert_eq!(16, keys.len());
@@ -583,15 +642,37 @@ mod tests {
     let mut ctx_b: FeelContext = Default::default();
     ctx_b.set_entry(&name_d, Value::Boolean(false));
     let mut ctx_a: FeelContext = Default::default();
-    ctx_a.set_entry(&name_a, Value::Number(1.0));
-    ctx_a.set_entry(&name_b, Value::Number(2.0));
+    ctx_a.set_entry(&name_a, value_number!(1));
+    ctx_a.set_entry(&name_b, value_number!(2));
     ctx_a.set_entry(&name_c, ctx_b.into());
     let keys = ctx_a.flatten_keys();
     assert_eq!(5, keys.len());
     assert!(keys.contains("b"));
     assert!(keys.contains("lorem ipsum dolor sit amet"));
-    assert!(keys.contains("now let ' s go to the next paragraph"));
-    assert!(keys.contains("now let ' s go to the next paragraph . Curabitur rhoncus + sodales odio in fringilla"));
-    assert!(keys.contains("Curabitur rhoncus + sodales odio in fringilla"));
+    assert!(keys.contains("now let's go to the next paragraph"));
+    assert!(keys.contains("now let's go to the next paragraph . Curabitur rhoncus+sodales odio in fringilla"));
+    assert!(keys.contains("Curabitur rhoncus+sodales odio in fringilla"));
+  }
+
+  #[test]
+  fn test_context_set_null() {
+    let name_a = Name::from("a");
+    let mut ctx: FeelContext = Default::default();
+    ctx.set_null(name_a);
+    assert_eq!(r#"{a: null}"#, ctx.to_string());
+  }
+
+  #[test]
+  fn test_get_entries() {
+    let mut ctx: FeelContext = Default::default();
+    assert_eq!(0, ctx.get_entries().len());
+    let name_a = Name::new(&["a"]);
+    ctx.set_entry(&name_a, value_number!(1));
+    assert_eq!(1, ctx.get_entries().len());
+    assert_eq!(vec![(&name_a, &value_number!(1))], ctx.get_entries());
+    let name_b = Name::new(&["b"]);
+    ctx.set_entry(&name_b, value_number!(2));
+    assert_eq!(2, ctx.get_entries().len());
+    assert_eq!(vec![(&name_a, &value_number!(1)), (&name_b, &value_number!(2))], ctx.get_entries());
   }
 }
