@@ -17,9 +17,11 @@
 use self::errors::*;
 use crate::temporal::ym_duration::FeelYearsAndMonthsDuration;
 use crate::temporal::{after, after_or_equal, before, before_or_equal, between, equal, weekday, FeelDateTime, FeelTime};
+use crate::FeelNumber;
 use chrono::{DateTime, Datelike, FixedOffset, Local};
 use dmntk_common::DmntkError;
 use regex::Regex;
+use std::cmp::Ordering;
 use std::convert::{TryFrom, TryInto};
 
 lazy_static! {
@@ -64,28 +66,49 @@ impl TryFrom<&str> for FeelDate {
   }
 }
 
-impl TryFrom<(f64, f64, f64)> for FeelDate {
+impl TryFrom<(FeelNumber, FeelNumber, FeelNumber)> for FeelDate {
   type Error = DmntkError;
   /// Converts a tuple of numbers into [FeelDate].
-  fn try_from(value: (f64, f64, f64)) -> Result<Self, Self::Error> {
-    let year = value.0.trunc() as i32;
-    if value.1 > 0.0 && value.2 > 0.0 {
-      let month = value.1.trunc() as u8;
-      let day = value.2.trunc() as u8;
+  fn try_from(value: (FeelNumber, FeelNumber, FeelNumber)) -> Result<Self, Self::Error> {
+    let year = value.0.into();
+    if value.1 > FeelNumber::zero() && value.2 > FeelNumber::zero() {
+      let month = value.1.into();
+      let day = value.2.into();
       if is_valid_date(year, month, day) {
         return Ok(Self(year, month, day));
       }
     }
-    Err(invalid_date(value.0, value.1, value.2))
+    Err(invalid_date(value.0.into(), value.1.into(), value.2.into()))
   }
 }
 
-impl std::cmp::PartialEq for FeelDate {
+impl PartialEq for FeelDate {
+  ///
   fn eq(&self, other: &Self) -> bool {
     self.0 == other.0 && self.1 == other.1 && self.2 == other.2
   }
 }
 
+impl PartialOrd for FeelDate {
+  ///
+  fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+    if self == other {
+      return Some(Ordering::Equal);
+    }
+    if let Some(before) = self.before(other) {
+      if before {
+        return Some(Ordering::Less);
+      }
+    }
+    if let Some(after) = self.after(other) {
+      if after {
+        return Some(Ordering::Greater);
+      }
+    }
+    None
+  }
+}
+
 impl TryFrom<FeelDate> for DateTime<FixedOffset> {
   type Error = DmntkError;
   /// Converts the `FEEL` date into chrono::DateTime<FixedOffset>.
@@ -100,7 +123,6 @@ impl FeelDate {
   pub fn new(year: i32, month: u8, day: u8) -> Self {
     Self(year, month, day)
   }
-
   ///
   pub fn new_opt(year: i32, month: u8, day: u8) -> Option<Self> {
     if is_valid_date(year, month, day) {
@@ -109,38 +131,37 @@ impl FeelDate {
       None
     }
   }
-
   ///
   pub fn today_local() -> Self {
     let today = Local::today();
     Self(today.year(), today.month() as u8, today.day() as u8)
   }
-
+  ///
   pub fn equal(&self, other: &Self) -> Option<bool> {
     let midnight = FeelTime::utc(0, 0, 0, 0);
     equal(&FeelDateTime(self.clone(), midnight.clone()), &FeelDateTime(other.clone(), midnight))
   }
-
+  ///
   pub fn before(&self, other: &Self) -> Option<bool> {
     let midnight = FeelTime::utc(0, 0, 0, 0);
     before(&FeelDateTime(self.clone(), midnight.clone()), &FeelDateTime(other.clone(), midnight))
   }
-
+  ///
   pub fn before_or_equal(&self, other: &Self) -> Option<bool> {
     let midnight = FeelTime::utc(0, 0, 0, 0);
     before_or_equal(&FeelDateTime(self.clone(), midnight.clone()), &FeelDateTime(other.clone(), midnight))
   }
-
+  ///
   pub fn after(&self, other: &Self) -> Option<bool> {
     let midnight = FeelTime::utc(0, 0, 0, 0);
     after(&FeelDateTime(self.clone(), midnight.clone()), &FeelDateTime(other.clone(), midnight))
   }
-
+  ///
   pub fn after_or_equal(&self, other: &Self) -> Option<bool> {
     let midnight = FeelTime::utc(0, 0, 0, 0);
     after_or_equal(&FeelDateTime(self.clone(), midnight.clone()), &FeelDateTime(other.clone(), midnight))
   }
-
+  ///
   pub fn between(&self, left: &Self, right: &Self, left_closed: bool, right_closed: bool) -> Option<bool> {
     let midnight = FeelTime::utc(0, 0, 0, 0);
     between(
@@ -151,7 +172,7 @@ impl FeelDate {
       right_closed,
     )
   }
-
+  ///
   pub fn ym_duration(&self, other: &FeelDate) -> FeelYearsAndMonthsDuration {
     let mut months;
     if self.0 < other.0 {
@@ -168,28 +189,29 @@ impl FeelDate {
     }
     FeelYearsAndMonthsDuration::new_m(months)
   }
-
+  ///
   pub fn year(&self) -> i32 {
     self.0
   }
-
+  ///
   pub fn month(&self) -> u8 {
     self.1
   }
-
+  ///
   pub fn day(&self) -> u8 {
     self.2
   }
-
+  ///
   pub fn weekday(&self) -> Option<u32> {
     weekday(&FeelDateTime(self.clone(), FeelTime::utc(0, 0, 0, 0)))
   }
-
+  ///
   pub fn as_tuple(&self) -> (i32, u32, u32) {
     (self.0, self.1 as u32, self.2 as u32)
   }
 }
 
+///
 pub fn is_valid_date(year: i32, month: u8, day: u8) -> bool {
   if DateTime::try_from(FeelDate(year, month, day)).is_ok() {
     return true;
@@ -202,10 +224,12 @@ pub fn is_valid_date(year: i32, month: u8, day: u8) -> bool {
   false
 }
 
+///
 pub fn is_leap_year(year: i32) -> bool {
   year % 4 == 0 && (year % 100 != 0 || year % 400 == 0)
 }
 
+///
 pub fn last_day_of_month(year: i32, month: u8) -> Option<u8> {
   match month {
     1 | 3 | 5 | 7 | 8 | 10 | 12 => Some(31),
@@ -223,10 +247,9 @@ pub mod errors {
   #[derive(Debug, PartialEq)]
   enum FeelDateError {
     InvalidDateLiteral(String),
-    InvalidDate(f64, f64, f64),
+    InvalidDate(i32, u8, u8),
   }
 
-  //TODO https://github.com/EngosSoftware/dmntk/issues/1
   impl From<FeelDateError> for DmntkError {
     fn from(e: FeelDateError) -> Self {
       DmntkError::new("FeelDateError", &format!("{}", e))
@@ -250,7 +273,7 @@ pub mod errors {
     FeelDateError::InvalidDateLiteral(literal).into()
   }
 
-  pub fn invalid_date(y: f64, m: f64, d: f64) -> DmntkError {
+  pub fn invalid_date(y: i32, m: u8, d: u8) -> DmntkError {
     FeelDateError::InvalidDate(y, m, d).into()
   }
 }
