@@ -72,7 +72,7 @@ impl std::str::FromStr for FeelTime {
   fn from_str(s: &str) -> Result<Self, Self::Err> {
     // parse the time from the provided string
     let time = parse_time_literal(s)?;
-    // even if parsing from string was successful, the time may be still invalid,
+    // even if parsing from string was successful, the time may still be invalid,
     // so check the time validity by converting to chrono::DateTime<FixedOffset>
     let _: DateTime<FixedOffset> = time.clone().try_into()?;
     // time is valid
@@ -100,33 +100,17 @@ impl TryFrom<FeelTime> for DateTime<FixedOffset> {
 }
 
 impl FeelTime {
-  pub fn new_hmso_opt(hour: f64, minute: f64, second: f64, offset: i64) -> Option<Self> {
-    if hour < 0.0 || minute < 0.0 || second < 0.0 {
-      return None;
-    }
-    let fractional = second.fract();
-    let nanos = if fractional > 0.0 { (fractional * 1e9).trunc() as u64 } else { 0u64 };
-    if is_valid_time(hour.trunc() as u8, minute.trunc() as u8, second.trunc() as u8) {
-      Some(Self(
-        hour.trunc() as u8,
-        minute.trunc() as u8,
-        second.trunc() as u8,
-        nanos,
-        FeelZone::new(offset),
-      ))
+  pub fn new_hmso_opt(hour: u8, minute: u8, second: u8, nano: u64, offset: i64) -> Option<Self> {
+    if is_valid_time(hour, minute, second) {
+      Some(Self(hour, minute, second, nano, FeelZone::new(offset)))
     } else {
       None
     }
   }
 
-  pub fn new_hms_opt(hour: f64, minute: f64, second: f64) -> Option<Self> {
-    if hour < 0.0 || minute < 0.0 || second < 0.0 {
-      return None;
-    }
-    let fractional = second.fract();
-    let nanos = if fractional > 0.0 { (fractional * 1e9).trunc() as u64 } else { 0u64 };
-    if is_valid_time(hour.trunc() as u8, minute.trunc() as u8, second.trunc() as u8) {
-      Some(Self(hour.trunc() as u8, minute.trunc() as u8, second.trunc() as u8, nanos, FeelZone::Local))
+  pub fn new_hms_opt(hour: u8, minute: u8, second: u8, nano: u64) -> Option<Self> {
+    if is_valid_time(hour, minute, second) {
+      Some(Self(hour, minute, second, nano, FeelZone::Local))
     } else {
       None
     }
@@ -281,7 +265,7 @@ impl std::ops::Sub<&FeelDateTime> for &FeelDateTime {
   type Output = Option<i64>;
   ///
   fn sub(self, rhs: &FeelDateTime) -> Option<i64> {
-    subtract(&self, rhs)
+    subtract(self, rhs)
   }
 }
 
@@ -295,7 +279,7 @@ impl TryFrom<FeelDateTime> for DateTime<FixedOffset> {
       FeelZone::Utc => Some(0),
       FeelZone::Local => get_local_offset(me_date_tuple, me_time_tuple),
       FeelZone::Offset(offset) => Some(*offset),
-      FeelZone::Zone(zone_name) => get_zone_offset(&zone_name, me_date_tuple, me_time_tuple),
+      FeelZone::Zone(zone_name) => get_zone_offset(zone_name, me_date_tuple, me_time_tuple),
     };
     if let Some(me_offset) = me_offset_opt {
       if let Some(me_date) = date_time_offset(me_date_tuple, me_time_tuple, me_offset) {
@@ -364,7 +348,7 @@ impl FeelDateTime {
   }
 
   pub fn ym_duration_1(&self, other: &FeelDate) -> FeelYearsAndMonthsDuration {
-    self.0.ym_duration(&other)
+    self.0.ym_duration(other)
   }
 
   pub fn year(&self) -> i32 {
@@ -396,11 +380,11 @@ impl FeelDateTime {
   }
 
   pub fn feel_time_offset(&self) -> Option<i32> {
-    feel_time_offset(&self)
+    feel_time_offset(self)
   }
 
   pub fn feel_time_zone(&self) -> Option<String> {
-    feel_time_zone(&self)
+    feel_time_zone(self)
   }
 }
 
@@ -486,7 +470,7 @@ fn compare(me: &FeelDateTime, other: &FeelDateTime) -> Option<Ordering> {
     FeelZone::Utc => Some(0),
     FeelZone::Local => get_local_offset(me_date_tuple, me_time_tuple),
     FeelZone::Offset(offset) => Some(*offset),
-    FeelZone::Zone(zone_name) => get_zone_offset(&zone_name, me_date_tuple, me_time_tuple),
+    FeelZone::Zone(zone_name) => get_zone_offset(zone_name, me_date_tuple, me_time_tuple),
   };
   let other_date_tuple = other.0.as_tuple();
   let other_time_tuple = ((other.1).0 as u32, (other.1).1 as u32, (other.1).2 as u32, (other.1).3 as u32);
@@ -494,7 +478,7 @@ fn compare(me: &FeelDateTime, other: &FeelDateTime) -> Option<Ordering> {
     FeelZone::Utc => Some(0),
     FeelZone::Local => get_local_offset(other_date_tuple, other_time_tuple),
     FeelZone::Offset(offset) => Some(*offset),
-    FeelZone::Zone(zone_name) => get_zone_offset(&zone_name, other_date_tuple, other_time_tuple),
+    FeelZone::Zone(zone_name) => get_zone_offset(zone_name, other_date_tuple, other_time_tuple),
   };
   if let Some((me_offset, other_offset)) = me_offset_opt.zip(other_offset_opt) {
     let me_date_opt = date_time_offset(me_date_tuple, me_time_tuple, me_offset);
@@ -513,7 +497,7 @@ pub fn subtract(me: &FeelDateTime, other: &FeelDateTime) -> Option<i64> {
     FeelZone::Utc => Some(0),
     FeelZone::Local => get_local_offset(me_date_tuple, me_time_tuple),
     FeelZone::Offset(offset) => Some(*offset),
-    FeelZone::Zone(zone_name) => get_zone_offset(&zone_name, me_date_tuple, me_time_tuple),
+    FeelZone::Zone(zone_name) => get_zone_offset(zone_name, me_date_tuple, me_time_tuple),
   };
   let other_date_tuple = other.0.as_tuple();
   let other_time_tuple = ((other.1).0 as u32, (other.1).1 as u32, (other.1).2 as u32, (other.1).3 as u32);
@@ -521,7 +505,7 @@ pub fn subtract(me: &FeelDateTime, other: &FeelDateTime) -> Option<i64> {
     FeelZone::Utc => Some(0),
     FeelZone::Local => get_local_offset(other_date_tuple, other_time_tuple),
     FeelZone::Offset(offset) => Some(*offset),
-    FeelZone::Zone(zone_name) => get_zone_offset(&zone_name, other_date_tuple, other_time_tuple),
+    FeelZone::Zone(zone_name) => get_zone_offset(zone_name, other_date_tuple, other_time_tuple),
   };
   if let Some((me_offset, other_offset)) = me_offset_opt.zip(other_offset_opt) {
     let me_date_opt = date_time_offset(me_date_tuple, me_time_tuple, me_offset);
@@ -540,7 +524,7 @@ fn weekday(me: &FeelDateTime) -> Option<u32> {
     FeelZone::Utc => Some(0),
     FeelZone::Local => get_local_offset(me_date_tuple, me_time_tuple),
     FeelZone::Offset(offset) => Some(*offset),
-    FeelZone::Zone(zone_name) => get_zone_offset(&zone_name, me_date_tuple, me_time_tuple),
+    FeelZone::Zone(zone_name) => get_zone_offset(zone_name, me_date_tuple, me_time_tuple),
   };
   if let Some(me_offset) = me_offset_opt {
     if let Some(me_date) = date_time_offset(me_date_tuple, me_time_tuple, me_offset) {
@@ -557,7 +541,7 @@ fn feel_time_offset(me: &FeelDateTime) -> Option<i32> {
     FeelZone::Utc => Some(0),
     FeelZone::Local => None, // in FEEL semantic domain the local offset is treated as none
     FeelZone::Offset(offset) => Some(*offset),
-    FeelZone::Zone(zone_name) => get_zone_offset(&zone_name, me_date_tuple, me_time_tuple),
+    FeelZone::Zone(zone_name) => get_zone_offset(zone_name, me_date_tuple, me_time_tuple),
   };
   if let Some(me_offset) = me_offset_opt {
     return Some(me_offset);
@@ -652,6 +636,7 @@ mod tests {
   fn eq_time_utc(hour: u8, min: u8, sec: u8, s: &str) {
     let expected = FeelTime(hour, min, sec, 0, FeelZone::Utc);
     let actual = s.parse::<FeelTime>().expect("should not fail");
+    println!("{} {}", expected, actual);
     assert_eq!(Some(true), expected.equal(&actual));
   }
 
@@ -683,10 +668,15 @@ mod tests {
     eq_time_utc(16, 37, 9, "16:37:09Z");
     eq_time_utc(16, 37, 9, "16:37:09@Etc/UTC");
     eq_time_utc(16, 37, 9, "18:37:09@Africa/Johannesburg");
-    eq_time_utc(16, 37, 9, "17:37:09@Europe/London");
-    eq_time_utc(17, 37, 9, "10:37:09@America/Vancouver");
-    eq_time_utc(17, 37, 9, "13:37:09@America/New_York");
-    eq_time_utc(16, 37, 9, "18:37:09@Europe/Warsaw");
+    eq_time_utc(17, 37, 9, "17:37:09@Europe/London");
+
+    // eq_time_utc(17, 37, 9, "10:37:09@America/Vancouver"); // summer time in Vancouver
+    eq_time_utc(18, 37, 9, "10:37:09@America/Vancouver"); // winter time in Vancouver
+
+    // eq_time_utc(17, 37, 9, "13:37:09@America/New_York"); // summer time in New York
+    eq_time_utc(18, 37, 9, "13:37:09@America/New_York"); // winter time in New York
+
+    eq_time_utc(17, 37, 9, "18:37:09@Europe/Warsaw");
   }
 
   #[test]
