@@ -14,26 +14,27 @@
  * SOFTWARE.
  */
 
-//! Definitions of `FEEL` types.
+//! `FEEL` types.
 
 use self::errors::*;
 use crate::context::FeelContext;
 use crate::names::Name;
-use crate::values::{Value, Values, VALUE_NULL};
-use dmntk_common::{DmntkError, Result, Stringify};
+use crate::value_null;
+use crate::values::{Value, Values};
+use dmntk_common::{DmntkError, Result};
 use std::collections::BTreeMap;
 use std::str::FromStr;
 
 pub const FEEL_TYPE_NAME_ANY: &str = "Any";
-const FEEL_TYPE_NAME_BOOLEAN: &str = "boolean";
-const FEEL_TYPE_NAME_DATE: &str = "date";
-const FEEL_TYPE_NAME_DATE_AND_TIME: &str = "date and time";
-const FEEL_TYPE_NAME_DAYS_AND_TIME_DURATION: &str = "days and time duration";
-const FEEL_TYPE_NAME_NULL: &str = "Null";
-const FEEL_TYPE_NAME_NUMBER: &str = "number";
-const FEEL_TYPE_NAME_STRING: &str = "string";
-const FEEL_TYPE_NAME_TIME: &str = "time";
-const FEEL_TYPE_NAME_YEARS_AND_MONTHS_DURATION: &str = "years and months duration";
+pub const FEEL_TYPE_NAME_BOOLEAN: &str = "boolean";
+pub const FEEL_TYPE_NAME_DATE: &str = "date";
+pub const FEEL_TYPE_NAME_DATE_AND_TIME: &str = "date and time";
+pub const FEEL_TYPE_NAME_DAYS_AND_TIME_DURATION: &str = "days and time duration";
+pub const FEEL_TYPE_NAME_NULL: &str = "Null";
+pub const FEEL_TYPE_NAME_NUMBER: &str = "number";
+pub const FEEL_TYPE_NAME_STRING: &str = "string";
+pub const FEEL_TYPE_NAME_TIME: &str = "time";
+pub const FEEL_TYPE_NAME_YEARS_AND_MONTHS_DURATION: &str = "years and months duration";
 
 #[derive(Debug, Clone, PartialEq)]
 pub enum FeelType {
@@ -41,51 +42,77 @@ pub enum FeelType {
   Any,
   /// Type representing a `boolean` value.
   Boolean,
-  /// Type representing `context` values.
-  Context(BTreeMap<Name, FeelType>),
-  /// Type representing `date` value.
+  /// Type representing a `context` value.
+  Context(
+    /// Types of context entries.
+    BTreeMap<Name, FeelType>,
+  ),
+  /// Type representing a `date` value.
   Date,
-  /// Type representing `date and time` value.
+  /// Type representing a`date and time` value.
   DateTime,
-  /// Type representing `days and time duration` value.
+  /// Type representing a `days and time duration` value.
   DaysAndTimeDuration,
-  /// Type representing `function` values.
-  Function(Vec<FeelType>, Box<FeelType>),
-  /// Type representing `list` values.
+  /// Type representing a `function` value.
+  Function(
+    /// List of types of the function's parameters.
+    Vec<FeelType>,
+    /// Type of the function's result.
+    Box<FeelType>,
+  ),
+  /// Type representing a `list` of values.
   List(Box<FeelType>),
-  /// Type representing `null` value.
+  /// Type representing a `null` value.
   Null,
-  /// Type representing `number` value.
+  /// Type representing a `number` value.
   Number,
-  /// Type representing `range` values.
+  /// Type representing a `range` values.
   Range(Box<FeelType>),
-  /// Type representing `string` value.
+  /// Type representing a `string` value.
   String,
-  /// Type representing `time` value.
+  /// Type representing a `time` value.
   Time,
-  /// Type representing `years and months duration` value.
+  /// Type representing a `years and months duration` value.
   YearsAndMonthsDuration,
 }
 
-impl Stringify for FeelType {
-  fn stringify(&self) -> String {
+impl std::fmt::Display for FeelType {
+  fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
     match self {
-      FeelType::Any => FEEL_TYPE_NAME_ANY,
-      FeelType::Boolean => FEEL_TYPE_NAME_BOOLEAN,
-      FeelType::Context(_) => "context tbd",
-      FeelType::Date => FEEL_TYPE_NAME_DATE,
-      FeelType::DateTime => FEEL_TYPE_NAME_DATE_AND_TIME,
-      FeelType::DaysAndTimeDuration => FEEL_TYPE_NAME_DAYS_AND_TIME_DURATION,
-      FeelType::Function(_, _) => "function type (tbd)",
-      FeelType::List(_) => "list type (tbd)",
-      FeelType::Null => FEEL_TYPE_NAME_NULL,
-      FeelType::Number => FEEL_TYPE_NAME_NUMBER,
-      FeelType::Range(_) => "range type (tbd)",
-      FeelType::String => FEEL_TYPE_NAME_STRING,
-      FeelType::Time => FEEL_TYPE_NAME_TIME,
-      FeelType::YearsAndMonthsDuration => FEEL_TYPE_NAME_YEARS_AND_MONTHS_DURATION,
+      FeelType::Any => write!(f, "{}", FEEL_TYPE_NAME_ANY),
+      FeelType::Boolean => write!(f, "{}", FEEL_TYPE_NAME_BOOLEAN),
+      FeelType::Context(entries) => {
+        let entries_str = entries
+          .iter()
+          .map(|(entry_name, entry_type)| format!("{}: {}", entry_name, entry_type))
+          .collect::<Vec<String>>()
+          .join(", ");
+        write!(f, "context<{}>", entries_str)
+      }
+      FeelType::Date => write!(f, "{}", FEEL_TYPE_NAME_DATE),
+      FeelType::DateTime => write!(f, "{}", FEEL_TYPE_NAME_DATE_AND_TIME),
+      FeelType::DaysAndTimeDuration => write!(f, "{}", FEEL_TYPE_NAME_DAYS_AND_TIME_DURATION),
+      FeelType::Function(parameter_types, result_type) => {
+        let parameter_types_str = parameter_types
+          .iter()
+          .map(|parameter_type| format!("{}", parameter_type))
+          .collect::<Vec<String>>()
+          .join(", ");
+        let result_type_str = result_type.to_string();
+        write!(f, "function<{}>->{}", parameter_types_str, result_type_str)
+      }
+      FeelType::List(item_type) => {
+        write!(f, "list<{}>", item_type)
+      }
+      FeelType::Null => write!(f, "{}", FEEL_TYPE_NAME_NULL),
+      FeelType::Number => write!(f, "{}", FEEL_TYPE_NAME_NUMBER),
+      FeelType::Range(range_type) => {
+        write!(f, "range<{}>", range_type)
+      }
+      FeelType::String => write!(f, "{}", FEEL_TYPE_NAME_STRING),
+      FeelType::Time => write!(f, "{}", FEEL_TYPE_NAME_TIME),
+      FeelType::YearsAndMonthsDuration => write!(f, "{}", FEEL_TYPE_NAME_YEARS_AND_MONTHS_DURATION),
     }
-    .to_string()
   }
 }
 
@@ -109,7 +136,18 @@ impl FromStr for FeelType {
   }
 }
 
-/// Returns `true` when the specified name is a built-in type name.
+impl From<&Name> for FeelType {
+  /// Converts a `FEEL` name to built-in type.
+  fn from(name: &Name) -> Self {
+    if let Ok(feel_type) = Self::from_str(name.to_string().as_str()) {
+      feel_type
+    } else {
+      FeelType::Any
+    }
+  }
+}
+
+/// Returns `true` when the specified type name is a built-in type.
 pub fn is_built_in_type_name(name: &str) -> bool {
   matches!(
     name,
@@ -127,11 +165,81 @@ pub fn is_built_in_type_name(name: &str) -> bool {
 }
 
 impl FeelType {
+  /// When a value appears in a certain context, it must be compatible
+  /// with a type expected in that context, called the target type.
+  /// After the type of the value is known, an implicit conversion
+  /// from the type of the value to the target type can be performed.
+  /// If an implicit conversion is mandatory but it cannot be performed,
+  /// the result is null.
+  ///
+  /// There are several possible type conversions:
+  ///
+  /// - to singleton list:
+  ///
+  ///      When the type of the value is `T` and the target type is `List<T>`,
+  ///      the simple value is converted to a singleton list.
+  ///
+  /// - from singleton list:
+  ///
+  ///      When the type of the value is List<T>, and the value is a singleton list
+  ///      and the target type is T, the value is converted by unwrapping the first element.
+  ///
+  /// - conforms to:
+  ///
+  ///      When the type of the value is T1, the target type is T2, and T1 conforms to T2,
+  ///      the value remains unchanged. Otherwise the result is null.
+  ///
+  /// All these conversion rules are implemented in this function.
+  ///
+  pub fn coerced(&self, actual_value: &Value) -> Value {
+    // conforms to
+    if actual_value.type_of().is_conformant(self) {
+      return actual_value.clone();
+    }
+    match self {
+      // to singleton list
+      FeelType::List(target_type) => {
+        if actual_value.type_of().is_conformant(target_type) {
+          return Value::List(Values::new(vec![actual_value.clone()]));
+        }
+      }
+      // from singleton list
+      target_type => {
+        if let FeelType::List(actual_type) = actual_value.type_of() {
+          if actual_type.is_conformant(target_type) {
+            if let Value::List(values) = actual_value {
+              if values.len() == 1 {
+                return values.as_vec()[0].clone();
+              }
+            }
+          }
+        }
+      }
+    }
+    value_null!()
+  }
+  ///
+  pub fn get_conformant_value(&self, actual_value: &Value) -> Value {
+    let actual_type = actual_value.type_of();
+    if actual_type.is_conformant(self) {
+      if let Ok(expected_value) = self.get_value_checked(actual_value) {
+        expected_value
+      } else {
+        value_null!("actual value {} does not match expected type {}", actual_value, self.to_string())
+      }
+    } else {
+      value_null!(
+        "expected type {} is not conformant with actual type {}",
+        self.to_string(),
+        actual_type.to_string()
+      )
+    }
+  }
   /// Returns a new value cloned from provided value, and retrieved with type checking.
-  pub fn retrieve_value_with_type_checking(&self, value: &Value) -> Result<Value> {
+  pub fn get_value_checked(&self, value: &Value) -> Result<Value> {
     if let Value::Null(_) = value {
       // `null` value is conformant with all types
-      return Ok(VALUE_NULL);
+      return Ok(value_null!());
     }
     match self {
       FeelType::Any => {
@@ -147,7 +255,7 @@ impl FeelType {
           let mut result = FeelContext::default();
           for (name, entry_type) in entries {
             if let Some(entry_value) = context.get_entry(name) {
-              result.set_entry(name, entry_type.retrieve_value_with_type_checking(entry_value)?);
+              result.set_entry(name, entry_type.get_value_checked(entry_value)?);
             }
           }
           return Ok(Value::Context(result));
@@ -169,22 +277,22 @@ impl FeelType {
         }
       }
       FeelType::Function(_, _) => {
-        if let Value::FunctionDefinition(_, _) = value {
+        if let Value::FunctionDefinition { .. } = value {
           return Ok(value.clone());
         }
       }
       FeelType::List(items_type) => {
         if let Value::List(items) = value {
           let mut result = vec![];
-          for item in items.to_vec() {
-            result.push(items_type.retrieve_value_with_type_checking(&item)?);
+          for item in items.as_vec() {
+            result.push(items_type.get_value_checked(item)?);
           }
           return Ok(Value::List(Values::new(result)));
         }
       }
       FeelType::Null => {
         if let Value::Null(_) = value {
-          return Ok(VALUE_NULL);
+          return Ok(value_null!());
         }
       }
       FeelType::Number => {
@@ -213,8 +321,9 @@ impl FeelType {
         }
       }
     }
-    Err(invalid_value_for_retrieving_using_feel_type(&self.stringify(), &value.stringify()))
+    Err(invalid_value_for_retrieving_using_feel_type(&self.to_string(), &value.to_string()))
   }
+
   /// Returns `true` when this type is a simple `FEEL` type.
   pub fn is_simple_built_in_type(&self) -> bool {
     matches!(
@@ -239,11 +348,11 @@ impl FeelType {
   pub fn range(elements_type: &FeelType) -> FeelType {
     FeelType::Range(Box::new(elements_type.clone()))
   }
-  /// Creates a `context` type with specified entries' types.
+  /// Creates a `context` type with specified entries.
   pub fn context(entries_types: &[(&Name, &FeelType)]) -> FeelType {
     FeelType::Context(entries_types.iter().map(|(name, typ)| ((*name).clone(), (*typ).clone())).collect())
   }
-  /// Creates a `function` type with specified parameter types and return type.
+  /// Creates a `function` type with specified parameter types and result type.
   pub fn function(parameter_types: &[FeelType], result_type: &FeelType) -> FeelType {
     FeelType::Function(parameter_types.iter().map(|typ| (*typ).clone()).collect(), Box::new((*result_type).clone()))
   }
@@ -426,21 +535,22 @@ mod tests {
   use crate::names::Name;
   use crate::types::is_built_in_type_name;
   use crate::types::FeelType::{self, Any, Number};
-  use crate::values::Value;
+  use crate::values::{Value, Values};
+  use crate::{value_number, FeelNumber};
 
   #[test]
-  fn test_retrieve_value_with_type_checking() {
+  fn test_get_value_checked() {
     let t_any = FeelType::Any;
     let t_boolean = FeelType::Boolean;
     let v_null = Value::Null(None);
     let v_boolean = Value::Boolean(true);
     let v_string = Value::String("hello".to_owned());
-    assert_eq!(Value::Null(None), t_any.retrieve_value_with_type_checking(&v_null).unwrap());
-    assert_eq!(Value::Boolean(true), t_any.retrieve_value_with_type_checking(&v_boolean).unwrap());
-    assert!(t_boolean.retrieve_value_with_type_checking(&v_string).is_err());
+    assert_eq!("null", t_any.get_value_checked(&v_null).unwrap().to_string());
+    assert_eq!("true", t_any.get_value_checked(&v_boolean).unwrap().to_string());
+    assert!(t_boolean.get_value_checked(&v_string).is_err());
     assert_eq!(
       r#"TypesError: invalid value for retrieving with type check, type = `boolean`, value = `"hello"`"#,
-      format!("{}", t_boolean.retrieve_value_with_type_checking(&v_string).err().unwrap()).as_str()
+      format!("{}", t_boolean.get_value_checked(&v_string).err().unwrap()).as_str()
     );
   }
 
@@ -961,4 +1071,74 @@ mod tests {
     assert!(!is_built_in_type_name("context"));
     assert!(!is_built_in_type_name("function"));
   }
+
+  #[test]
+  fn test_type_stringify() {
+    let name_a = Name::from("a");
+    let name_b = Name::from("b");
+    let name_c = Name::from("c");
+    let t_any = FeelType::Any;
+    let t_null = FeelType::Null;
+    let t_string = FeelType::String;
+    let t_number = FeelType::Number;
+    let t_boolean = FeelType::Boolean;
+    let t_date = FeelType::Date;
+    let t_time = FeelType::Time;
+    let t_date_time = FeelType::DateTime;
+    let t_dt_duration = FeelType::DaysAndTimeDuration;
+    let t_ym_duration = FeelType::YearsAndMonthsDuration;
+    let t_function_a = FeelType::function(&[Number, Number], &t_number);
+    let t_function_b = FeelType::function(&[], &t_any);
+    let t_function_c = FeelType::function(&[], &t_string);
+    let t_list_a = FeelType::List(Box::new(FeelType::Boolean));
+    let t_list_b = FeelType::List(Box::new(FeelType::Number));
+    let t_range_a = FeelType::range(&t_number);
+    let t_range_b = FeelType::range(&t_date);
+    let t_context_a = FeelType::context(&[(&name_a, &t_number)]);
+    let t_context_b = FeelType::context(&[(&name_a, &t_number), (&name_b, &t_boolean)]);
+    let t_context_c = FeelType::context(&[(&name_a, &t_number), (&name_b, &t_boolean), (&name_c, &t_string)]);
+    assert_eq!("Any", t_any.to_string());
+    assert_eq!("Null", t_null.to_string());
+    assert_eq!("number", t_number.to_string());
+    assert_eq!("boolean", t_boolean.to_string());
+    assert_eq!("date", t_date.to_string());
+    assert_eq!("time", t_time.to_string());
+    assert_eq!("date and time", t_date_time.to_string());
+    assert_eq!("days and time duration", t_dt_duration.to_string());
+    assert_eq!("years and months duration", t_ym_duration.to_string());
+    assert_eq!("string", t_string.to_string());
+    assert_eq!("function<number, number>->number", t_function_a.to_string());
+    assert_eq!("function<>->Any", t_function_b.to_string());
+    assert_eq!("function<>->string", t_function_c.to_string());
+    assert_eq!("list<boolean>", t_list_a.to_string());
+    assert_eq!("list<number>", t_list_b.to_string());
+    assert_eq!("range<number>", t_range_a.to_string());
+    assert_eq!("range<date>", t_range_b.to_string());
+    assert_eq!("context<a: number>", t_context_a.to_string());
+    assert_eq!("context<a: number, b: boolean>", t_context_b.to_string());
+    assert_eq!("context<a: number, b: boolean, c: string>", t_context_c.to_string());
+  }
+
+  #[test]
+  fn test_get_coerced() {
+    let t_any = FeelType::Any;
+    let t_number = FeelType::Number;
+    let t_string = FeelType::String;
+    let t_list_number = FeelType::List(Box::new(t_number.clone()));
+    let t_list_string = FeelType::List(Box::new(t_string.clone()));
+    let t_list_any = FeelType::List(Box::new(t_any));
+    let v_number = value_number!(10, 0);
+    let v_string = Value::String("a".into());
+    let v_list_number_1 = Value::List(Values::new(vec![value_number!(1, 0)]));
+    let v_list_number_2 = Value::List(Values::new(vec![value_number!(1, 0), value_number!(2, 0)]));
+    let v_list_string_1 = Value::List(Values::new(vec![Value::String("A".to_string())]));
+    assert_eq!(r#"[10]"#, t_list_number.coerced(&v_number).to_string());
+    assert_eq!(r#"[10]"#, t_list_any.coerced(&v_number).to_string());
+    assert_eq!(r#"1"#, t_number.coerced(&v_list_number_1).to_string());
+    assert_eq!(r#"null"#, t_number.coerced(&v_list_number_2).to_string());
+    assert_eq!(r#"null"#, t_number.coerced(&v_list_string_1).to_string());
+    assert_eq!(r#"["a"]"#, t_list_string.coerced(&v_string).to_string());
+    assert_eq!(r#"["a"]"#, t_list_any.coerced(&v_string).to_string());
+    assert_eq!(r#""A""#, t_string.coerced(&v_list_string_1).to_string());
+  }
 }
